SystemOrganization addCategory: #'Mayur-Sachin-Ajith'!

Object subclass: #Employee
	instanceVariableNames: 'name transactions salary earned paid withholding accruedVacation taxRule'
	classVariableNames: 'HeadTaxRate MarriedSeparateTaxRate MarriedTaxRate SingleTaxRate'
	poolDictionaries: ''
	category: 'Mayur-Sachin-Ajith'!

!Employee commentStamp: 'm3r 6/6/2008 22:19' prior: 0!
I represent an employee in a payroll system.  Thus, I keep track of the number of hours worked,
the wages paid, the vacation accrued, the taxes withheld, and so on.  My values change only
when transactions are posted to me.  Transactions are subclasses of EmployeeTransaction.

Variables
	name 				<String>
	transactions 		<Collection of Transactions, sorted by date>
	salary 				<ValueWithHistory>
	earned  			<ValueWithHistory>
	paid  				<ValueWithHistory>
	withholding  		<ValueWithHistory>
	accruedVacation  	<ValueWithHistory>
	taxRule 			<TaxRule>!

!Employee class methodsFor: 'class initialization' stamp: ''!
initialize
	SingleTaxRate := TaxRule new.
	SingleTaxRate breakPoint: 0 rate: 0.15;
		 breakPoint: 22750 rate: 0.28;
		 breakPoint: 55100 rate: 0.31;
		 breakPoint: 115000 rate: 0.36;
		 breakPoint: 250000 rate: 0.396.
	MarriedTaxRate := TaxRule new.
	MarriedTaxRate breakPoint: 0 rate: 0.15;
		 breakPoint: 38000 rate: 0.28;
		 breakPoint: 91850 rate: 0.31;
		 breakPoint: 140000 rate: 0.36;
		 breakPoint: 250000 rate: 0.396.
	MarriedSeparateTaxRate := TaxRule new.
	MarriedSeparateTaxRate breakPoint: 0 rate: 0.15;
		 breakPoint: 19500 rate: 0.28;
		 breakPoint: 45925 rate: 0.31;
		 breakPoint: 70000 rate: 0.36;
		 breakPoint: 125000 rate: 0.396.
	HeadTaxRate := TaxRule new.
	HeadTaxRate breakPoint: 0 rate: 0.15;
		 breakPoint: 30500 rate: 0.28;
		 breakPoint: 78700 rate: 0.31;
		 breakPoint: 127500 rate: 0.36;
		 breakPoint: 250000 rate: 0.396! !

!Employee methodsFor: 'actions' stamp: 'Ajith Herga 1/24/2013 18:04'!
changeSalaryFor: aDate to: dollarPerHour 
	self
		postTransaction: (SalaryChange
				date: aDate
				employee: self
				salary: dollarPerHour)! !

!Employee methodsFor: 'accessing' stamp: ''!
earned
	^ earned! !

!Employee methodsFor: 'private' stamp: ''!
federalTaxesOn: money assumingTotalIs: estimatedEarnings 
	^ money
		* (taxRule rateFor: estimatedEarnings)! !

!Employee methodsFor: 'posting' stamp: ''!
incrementEarned: anAmount 
	earned := earned + anAmount! !

!Employee methodsFor: 'posting' stamp: ''!
incrementPaid: anAmount 
	paid := paid + anAmount! !

!Employee methodsFor: 'posting' stamp: ''!
incrementWithholding: anAmount 
	withholding := withholding + anAmount! !

!Employee methodsFor: 'actions' stamp: 'Ajith Herga 1/24/2013 18:07'!
makePaycheckAt: aDate 
	| amountPaid amountWithheld |
	amountPaid := earned - paid - withholding.
	amountWithheld := self taxesOn: amountPaid at: aDate.
	amountPaid := amountPaid - amountWithheld.
	^ Paycheck
		date: aDate
		employee: self
		paid: amountPaid
		withheld: amountWithheld
		totalPaid: paid + amountPaid
		totalWithheld: withholding + amountWithheld! !

!Employee methodsFor: 'accessing' stamp: ''!
name
	^ name! !

!Employee methodsFor: 'initialize-release' stamp: ''!
named: aString 
	name := aString.
	transactions := OrderedCollection new.
	salary := 0.
	earned := 0.
	paid := 0.
	withholding := 0.
	accruedVacation := 0.
	taxRule := SingleTaxRate! !

!Employee methodsFor: 'accessing' stamp: ''!
paid
	^ paid! !

!Employee methodsFor: 'actions' stamp: 'Ajith Herga 1/24/2013 18:06'!
postTimeCardFor: aDate hoursWorked: hoursW vacation: hoursV 
	self
		postTransaction: (Timecard
				date: aDate
				employee: self
				worked: hoursW
				vacation: hoursV)! !

!Employee methodsFor: 'posting' stamp: ''!
postTransaction: aTransaction 
	aTransaction postTo: self.
	transactions add: aTransaction! !

!Employee methodsFor: 'accessing' stamp: ''!
salary
	^ salary! !

!Employee methodsFor: 'accessing' stamp: ''!
salary: anAmount 
	salary := anAmount! !

!Employee methodsFor: 'private' stamp: ''!
taxesOn: money at: date 
	| socialSecurity federalTaxes stateTaxes estimatedEarnings |
	socialSecurity := (0
				max: (money min: 60600 - earned))
				* 0.077.
	estimatedEarnings := earned + money * date day / date daysInYear.
	federalTaxes := self federalTaxesOn: money assumingTotalIs: estimatedEarnings.
	stateTaxes := money * 0.03.
	^ socialSecurity + federalTaxes + stateTaxes! !

Object subclass: #EmployeeTransaction
	instanceVariableNames: 'date employee'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mayur-Sachin-Ajith'!

!EmployeeTransaction commentStamp: 'm3r 6/6/2008 22:19' prior: 0!
I'm an abstract class of transactions that can be posted to an Employee.

My most important method is postTo:, which is implemented by subclasses
to send a message to the Employee to post the transaction.  The Employee
has all the information, so it has to be the one to implement posting, but the
postTo: method implements a standard interface so that clients of the
payroll system can always post a transaction by sending the postTransaction:
message to an Employee.

Variables
	date			<Date>   		Date at which transaction occured.
	employee	<Employee>

!

!EmployeeTransaction methodsFor: 'posting' stamp: ''!
post
	employee postTransaction: self! !

!EmployeeTransaction methodsFor: 'posting' stamp: ''!
postTo: anEmployee 
	self subclassResponsibility! !

EmployeeTransaction subclass: #Paycheck
	instanceVariableNames: 'amountPaid taxes totalPaid totalTaxes'
	classVariableNames: 'AmountFormat DateFormat'
	poolDictionaries: ''
	category: 'Mayur-Sachin-Ajith'!

!Paycheck commentStamp: '<historical>' prior: 0!
Transaction representing a payment to an employee.!

!Paycheck class methodsFor: 'instance creation' stamp: 'Ajith Herga 1/24/2013 18:00'!
date: aDate employee: anEmployee paid: amount1 withheld: amount2 totalPaid: amount3 totalWithheld: amount4 

    ^self new 
         date: aDate
         employee: anEmployee
         paid: amount1
         withheld: amount2
         totalPaid: amount3 
         totalWithheld: amount4! !

!Paycheck class methodsFor: 'class initialization' stamp: ''!
initialize
	^ self! !

!Paycheck methodsFor: 'initialize-release' stamp: ''!
date: aDate employee: anEmployee paid: amount1 withheld: amount2 totalPaid: amount3 totalWithheld: amount4 
	date := aDate.
	employee := anEmployee.
	amountPaid := amount1.
	taxes := amount2.
	totalPaid := amount3.
	totalTaxes := amount4! !

!Paycheck methodsFor: 'posting' stamp: ''!
postTo: anEmployee 
	anEmployee incrementPaid: amountPaid.
	anEmployee incrementWithholding: taxes! !

!Paycheck methodsFor: 'printing' stamp: ''!
printOnCheckStream: aStream 
	aStream cr; cr.
	aStream next: 40 put: Character space.
	date printOn: aStream.
	aStream cr.
	aStream
		nextPutAll: ((String new: 40 withAll: Character space)
				copyReplaceFrom: 1
				to: employee name size
				with: employee name).
	amountPaid printOn: aStream.
	aStream cr; cr; cr; cr.
	aStream nextPutAll: 'this month  '.
	aStream nextPutAll: ' amountPaid: $'.
	amountPaid printOn: aStream.
	aStream nextPutAll: ' taxes: $'.
	taxes printOn: aStream.
	aStream cr.
	aStream nextPutAll: 'year to date '.
	aStream nextPutAll: ' amountPaid: $'.
	totalPaid printOn: aStream.
	aStream nextPutAll: ' taxes: $'.
	totalTaxes printOn: aStream.
	aStream cr; cr! !

EmployeeTransaction subclass: #SalaryChange
	instanceVariableNames: 'newSalary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mayur-Sachin-Ajith'!

!SalaryChange commentStamp: '<historical>' prior: 0!
Transaction representing a change in salary, usually a raise.!

!SalaryChange class methodsFor: 'initialize-release' stamp: 'Ajith Herga 1/24/2013 18:02'!
date: aDate employee: anEmployee salary: dollarPerHour 
    ^self new date: aDate employee: anEmployee salary: dollarPerHour 
! !

!SalaryChange methodsFor: 'initialize-release' stamp: ''!
date: aDate employee: anEmployee salary: dollarPerHour 
	date := aDate.
	employee := anEmployee.
	newSalary := dollarPerHour! !

!SalaryChange methodsFor: 'posting' stamp: ''!
postTo: anEmployee 
	anEmployee salary: newSalary! !

EmployeeTransaction subclass: #Timecard
	instanceVariableNames: 'hoursWorked hoursVacation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mayur-Sachin-Ajith'!

!Timecard commentStamp: '<historical>' prior: 0!
Transaction representing the work that the employee has done.!

!Timecard class methodsFor: 'instance creation' stamp: 'Ajith Herga 1/24/2013 18:02'!
date: aDate employee: anEmployee worked: hoursW vacation: hoursV 
    ^self new date: aDate employee: anEmployee worked: hoursW vacation: hoursV! !

!Timecard methodsFor: 'initialize-release' stamp: ''!
date: aDate employee: anEmployee worked: hoursW vacation: hoursV 
	date := aDate.
	employee := anEmployee.
	hoursWorked := hoursW.
	hoursVacation := hoursV! !

!Timecard methodsFor: 'posting' stamp: ''!
postTo: anEmployee 
	| money overtime nonovertime |
	overtime := hoursWorked - 40 max: 0.
	nonovertime := hoursWorked min: 40.
	money := overtime * 1.5 + nonovertime * anEmployee salary.
	anEmployee incrementEarned: money! !

Object subclass: #NumberWithUnitsTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mayur-Sachin-Ajith'!

!NumberWithUnitsTestCase methodsFor: 'as yet unclassified' stamp: 'AjithHerga 2/4/2013 13:13'!
setUp
	"comment stating purpose of message"
    | dict1 dict2 num1 num2 num3 |
    dict1 := Dictionary new.
    dict1 at: #seconds put: 2.
    dict2 := Dictionary new.
    dict2 at: #meters put: 1.
    num1 := NumberWithUnits val: 4 unitDict: dict1.
    num2 := NumberWithUnits val: 5 unitDict: dict1.
    num3 := NumberWithUnits val: 6 unitDict: dict2.
    ! !

Object subclass: #PayrollSystem
	instanceVariableNames: 'employees paychecks companyName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mayur-Sachin-Ajith'!

!PayrollSystem commentStamp: '<historical>' prior: 0!
I am the top-level of the payroll system.  I keep track of the employees
and can "run the payroll", i.e. write paychecks for the employees.  To
run the payroll, send the messages
	makePaychecksAt: paydate	- to create the paycheck transactions
	printPaychecksOn: aStream	- to print them out 
	postPaychecks				- to record them in employees

Note that the employees are not marked with having been paid until
the paychecks are posted, so just creating the transactions or even
printing them is not enough.

Variables
	employees   <Dictionary from String to Employee>
	paychecks 	<Collection of Paycheck>	holds paychecks that have been created until they have been posted!

!PayrollSystem class methodsFor: 'instance creation' stamp: ''!
new
	^ super new initialize! !

!PayrollSystem methodsFor: 'accessing' stamp: ''!
addEmployee: anEmployee 
	employees at: anEmployee name put: anEmployee! !

!PayrollSystem methodsFor: 'accessing' stamp: ''!
employeeNamed: aName 
	^ employees at: aName! !

!PayrollSystem methodsFor: 'accessing' stamp: ''!
employeeNames
	^ employees keys asOrderedCollection! !

!PayrollSystem methodsFor: 'initialize-release' stamp: ''!
initialize
	employees := Dictionary new! !

!PayrollSystem methodsFor: 'actions' stamp: ''!
makePaychecksAt: aDate 
	paychecks := employees
				collect: [:eachEmployee | eachEmployee makePaycheckAt: aDate]! !

!PayrollSystem methodsFor: 'accessing' stamp: ''!
name
	^ companyName! !

!PayrollSystem methodsFor: 'accessing' stamp: ''!
name: aString 
	companyName := aString! !

!PayrollSystem methodsFor: 'actions' stamp: ''!
paycheckString
	| aStream |
	aStream := WriteStream
				on: (String new: 20).
	self printPaychecksOn: aStream.
	^ aStream contents! !

!PayrollSystem methodsFor: 'actions' stamp: ''!
postPaychecks
	paychecks
		do: [:each | each post].
	paychecks := #()! !

!PayrollSystem methodsFor: 'actions' stamp: ''!
printPaychecksOn: aStream 
	paychecks
		do: [:each | each printOnCheckStream: aStream]! !

Object subclass: #TaxRule
	instanceVariableNames: 'breakPoints rates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mayur-Sachin-Ajith'!

!TaxRule commentStamp: 'm3r 6/6/2008 22:20' prior: 0!
I represent a tax table.  Given a yearly taxable income, I can give the tax rate for it.
I store the tax table as a sequence of amounts, and the tax rate for that amount.
For example, in 1994 the tax rate for a single person (schedule X) was
          $0		15%
  		$22750	28%
  		$55100	31%
		$115000	36%
		$250000	39.6%

breakpoints  <SortedCollection of: integer>
rates	 	<OrderedCollection of: integer>!

!TaxRule class methodsFor: 'instance creation' stamp: ''!
new
	^ super new initialize! !

!TaxRule methodsFor: 'accessing' stamp: ''!
breakPoint: aNumber rate: anotherNumber 
	| index |
	index := breakPoints indexOfStartOfIntervalContaining: aNumber.
	breakPoints add: aNumber.
	rates add: anotherNumber beforeIndex: index + 1! !

!TaxRule methodsFor: 'initialize-release' stamp: ''!
initialize
	breakPoints := SortedSequence new.
	rates := OrderedCollection new! !

!TaxRule methodsFor: 'accessing' stamp: ''!
rateFor: aNumber 
	^ rates
		at: (breakPoints indexOfStartOfIntervalContaining: aNumber)! !

Number subclass: #NumberWithUnits
	instanceVariableNames: 'value units'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mayur-Sachin-Ajith'!

!NumberWithUnits class methodsFor: 'as yet unclassified' stamp: 'MayurSadavarte 2/4/2013 02:16'!
val: inValue unitDict: inUnits
	^ self new val: inValue  unitDict: inUnits .! !

!NumberWithUnits methodsFor: 'as yet unclassified' stamp: 'SachinKulkarni 2/4/2013 02:48'!
* aNumberWithUnits
	"comment stating purpose of message"
	
	(aNumberWithUnits isNumberWithUnits) ifFalse: [self error: 'Expecting NumberWithUnits']. 
	
	^NumberWithUnits val: ((self getValue)*(aNumberWithUnits getValue)) unitDict: ((self getUnits) * (aNumberWithUnits getUnits)).
	! !

!NumberWithUnits methodsFor: 'as yet unclassified' stamp: 'SachinKulkarni 2/4/2013 03:22'!
+ aNumberWithUnits
	(aNumberWithUnits isNumberWithUnits) ifFalse: [self error: 'Expecting NumberWithUnits'].
	((self getUnits) = (aNumberWithUnits getUnits)) ifFalse: [self error: 'Cannot add NumberWithUnits with different units'].
	^ NumberWithUnits val: ((self getValue) + (aNumberWithUnits getValue)) unitDict: (self getUnits).! !

!NumberWithUnits methodsFor: 'as yet unclassified' stamp: 'SachinKulkarni 2/4/2013 03:23'!
- aNumberWithUnits
	(aNumberWithUnits isNumberWithUnits) ifFalse: [self error: 'Expecting NumberWithUnits'].
	((self getUnits) = (aNumberWithUnits getUnits)) ifFalse: [self error: 'Cannot subtract NumberWithUnits with different units'].
	^NumberWithUnits val: ((self getValue) - (aNumberWithUnits getValue)) unitDict: (self getUnits).! !

!NumberWithUnits methodsFor: 'as yet unclassified' stamp: 'Ajith Herga 2/4/2013 02:59'!
/ aNumberWithUnits
	"comment stating purpose of message"

	(aNumberWithUnits isNumberWithUnits) ifFalse: [self error: 'Expecting NumberWithUnits'].
	^NumberWithUnits val: (((self getValue) asNumber) / ((aNumberWithUnits getValue) asNumber))
	                 unitDict: ((self getUnits) / (aNumberWithUnits getUnits))! !

!NumberWithUnits methodsFor: 'as yet unclassified' stamp: 'MayurSadavarte 2/4/2013 03:30'!
< aNumberWithUnits
	(aNumberWithUnits isNumberWithUnits ) ifFalse: [^self error: 'Cant execute '<' on two different classes'].
	(self getUnits = aNumberWithUnits getUnits ) ifFalse: [^self error: 'Cant compare values with different units'].
	^ self getValue < aNumberWithUnits getValue.! !

!NumberWithUnits methodsFor: 'as yet unclassified' stamp: 'MayurSadavarte 2/4/2013 02:37'!
= aNumberWithUnits
	(aNumberWithUnits isNumberWithUnits ) ifFalse: [^false].
	(self getUnits = aNumberWithUnits getUnits ) ifFalse: [^false].
	^ self getValue = aNumberWithUnits getValue.! !

!NumberWithUnits methodsFor: 'as yet unclassified' stamp: 'MayurSadavarte 2/4/2013 03:30'!
> aNumberWithUnits
	(aNumberWithUnits isNumberWithUnits ) ifFalse: [^self error: 'Cant execute '>' on two different classes'].
	(self getUnits = aNumberWithUnits getUnits ) ifFalse: [^self error: 'Cant compare values with different units'].
	^ self getValue < aNumberWithUnits getValue.
! !

!NumberWithUnits methodsFor: 'as yet unclassified' stamp: 'SachinKulkarni 2/4/2013 01:39'!
adaptToFraction: rcvr andSend: selector
	"If I am involved in arithmetic with a Fraction, convert us and evaluate exprBlock."
	^true! !

!NumberWithUnits methodsFor: 'as yet unclassified' stamp: 'SachinKulkarni 2/4/2013 01:39'!
adaptToInteger: rcvr andSend: selector
	"If I am involved in arithmetic with a Integer, convert us and evaluate exprBlock."
	^true! !

!NumberWithUnits methodsFor: 'as yet unclassified' stamp: 'MayurSadavarte 2/4/2013 02:17'!
getUnits 
	^ units ! !

!NumberWithUnits methodsFor: 'as yet unclassified' stamp: 'MayurSadavarte 2/4/2013 02:17'!
getValue 
	^ value ! !

!NumberWithUnits methodsFor: 'as yet unclassified' stamp: 'SachinKulkarni 2/4/2013 01:57'!
isNumberWithUnits
	^ true! !

!NumberWithUnits methodsFor: 'as yet unclassified' stamp: 'MayurSadavarte 2/4/2013 03:04'!
printOn: aStream base: aBase
	"This method should print a representation of the number for the given base,
	excluding the base prefix (and the letter r for radix)"
	self getValue printOn: aStream base: aBase .
	aStream space .
	self getUnits  printOn: aStream! !

!NumberWithUnits methodsFor: 'as yet unclassified' stamp: 'SachinKulkarni 2/4/2013 01:40'!
storeOn: aStream base: base
	"This method should print a representation of the number for the given base,
	including the base prefix (with letter r for radix)"
	
	^true! !

!NumberWithUnits methodsFor: 'as yet unclassified' stamp: 'MayurSadavarte 2/4/2013 02:17'!
val: inValue unitDict: inUnits
	value := inValue .
	units := inUnits .! !

SortedCollection variableSubclass: #SortedSequence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mayur-Sachin-Ajith'!

!SortedSequence methodsFor: 'accessing' stamp: ''!
indexOfStartOfIntervalContaining: anElement 
	self isEmpty
		ifTrue: [^ 0].
	^ (self indexForInserting: anElement)
		- firstIndex! !

Dictionary subclass: #Units
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mayur-Sachin-Ajith'!

!Units class methodsFor: 'as yet unclassified' stamp: 'Ajith Herga 1/31/2013 11:37'!
add: aName
	"Add named element to dictionary"

	| temporary variable names statements |
	^ Units new.! !

!Units class methodsFor: 'as yet unclassified' stamp: 'Ajith Herga 2/3/2013 20:21'!
addUnits: aDict
	"Add named element to dictionary"
	^ Units new addUnits: aDict.! !

!Units class methodsFor: 'as yet unclassified' stamp: 'Ajith Herga 1/31/2013 12:23'!
addUnits: aValue at: aName
	"Add named element to dictionary"
	^ Units new addUnits: aValue at: aName.! !

!Units methodsFor: 'as yet unclassified' stamp: 'Ajith Herga 2/3/2013 22:30'!
* aUnit
	"comment stating purpose of message"
	| dict1 |
    dict1 := Dictionary new.
	[self keysSortedSafely
				do: [:key |
					aUnit at: key ifPresent: [ dict1 at: key put: ((self at: key) + (aUnit at: key))].
					aUnit at: key ifAbsent: [ dict1 at: key put: (self at: key)]
					]].
	[aUnit keysSortedSafely 
	          do: [:key |
		           dict1 at: key ifAbsent: [ dict1 at: key put: (aUnit at: key)]]].
    ^Units new: dict1.! !

!Units methodsFor: 'as yet unclassified' stamp: 'Ajith Herga 2/3/2013 22:43'!
/ aUnit
	"comment stating purpose of message"
	| dict1 |
    dict1 := Dictionary new.
	[self keysSortedSafely
				do: [:key |
					aUnit at: key ifPresent: [ dict1 at: key put: ((self at: key) - (aUnit at: key))].
					aUnit at: key ifAbsent: [ dict1 at: key put: (self at: key)]
					]].
	[aUnit keysSortedSafely 
	          do: [:key |
		           dict1 at: key ifAbsent: [ dict1 at: key put: ((-1)*(aUnit at: key))]]].
    ^Units new: dict1.! !

!Units methodsFor: 'as yet unclassified' stamp: 'Ajith Herga 2/3/2013 20:54'!
= aUnits
	"comment stating purpose of message"
  (self class = aUnits class) ifFalse: [^false]. 
  (aUnits isUnits) ifFalse: [^ false].
  ^super = aUnits.! !

!Units methodsFor: 'as yet unclassified' stamp: 'Ajith Herga 2/3/2013 20:24'!
addUnits: aDict
"Add a unit to the dictionary, include the value"
 "  self at: aName ifPresent: [:x | self at: aName put: ((self at: aName) + aValue)]."
	[aDict keysSortedSafely
				do: [:key | self at: key put: (aDict  at: key)]].! !

!Units methodsFor: 'as yet unclassified' stamp: 'Ajith Herga 2/3/2013 19:20'!
addUnits: aValue at: aName
	"Add a unit to the dictionary, include the value"
 "  self at: aName ifPresent: [:x | self at: aName put: ((self at: aName) + aValue)]."
    self at: aName put: aValue
! !

!Units methodsFor: 'as yet unclassified' stamp: 'Ajith Herga 2/3/2013 19:33'!
isUnits
	"comment stating purpose of messages"
	^true! !

!Units methodsFor: 'as yet unclassified' stamp: 'Ajith Herga 2/3/2013 19:37'!
printOn
	"comment stating purpose of message"

	| temporary variable names |
	'OK'! !

!Units methodsFor: 'as yet unclassified' stamp: 'Ajith Herga 2/3/2013 22:08'!
printOn: aStream
	"comment stating purpose of message"
	aStream nextPut: $(.
	self size > 100
		ifTrue: [aStream nextPutAll: 'size '.
			self size printOn: aStream]
		ifFalse: [self keysSortedSafely
				do: [:key | aStream print: key asString;
						 nextPutAll: '^';				
						 print: (self at: key);
						 space]].
	aStream nextPut: $)! !

TestCase subclass: #EmployeeTestCase
	instanceVariableNames: 'susan day1 normalMonth aLittleOvertime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mayur-Sachin-Ajith'!

!EmployeeTestCase methodsFor: 'Running' stamp: ''!
setUp
	day1 := Date
				newDay: 11
				month: 7
				year: 2001.
	susan := Employee new named: 'Susan'.
	normalMonth := #(#(40 0) #(40 0) #(40 0) #(40 0) ).
	aLittleOvertime := #(#(40 0) #(50 0) #(40 0) #(40 0) )! !

!EmployeeTestCase methodsFor: 'Testing' stamp: ''!
testChangeSalary
	susan changeSalaryFor: day1 to: 20.
	self assert: susan salary = 20.
	day1 addDays: 7.
	susan changeSalaryFor: day1 to: 25.
	self assert: susan salary = 25! !

TestCase subclass: #PayrollSystemTestCase
	instanceVariableNames: 'myPayroll day1 susan george normalMonth aLittleOvertime ajith day2 john day3 day4'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mayur-Sachin-Ajith'!

!PayrollSystemTestCase methodsFor: 'private' stamp: ''!
postTimeCards: collection to: anEmployee 
	| timecardDay |
	timecardDay := day1.
	collection
		do: [:each | 
			anEmployee
				postTimeCardFor: timecardDay
				hoursWorked: each first
				vacation: each last.
			timecardDay := timecardDay addDays: 7]! !

!PayrollSystemTestCase methodsFor: 'Running' stamp: 'Ajith Herga 1/31/2013 00:22'!
setUp
	myPayroll := PayrollSystem new name: 'ACME'.
	day1 := Date newDay: 11 month: 7 year: 2001.
	day2 := Date newDay: 1 month: 5 year: 1999.
   day3 := Date newDay: 5 month: 5 year: 1999.
   day4 := Date newDay: 12 month: 5 year: 1999.
   john := Employee new named: 'John Doe'.
   john changeSalaryFor: day2 to: 20. 
	susan := Employee new named: 'Susan'.
	susan changeSalaryFor: day1 to: 20.
	george := Employee new named: 'George'.
	george changeSalaryFor: day1 to: 12.
	ajith := Employee new named: 'Ajith'.
	ajith changeSalaryFor: day1 to: 21. 
	normalMonth := #(#(40 0) #(40 0) #(40 0) #(40 0) ).
	aLittleOvertime := #(#(40 0) #(50 0) #(40 0) #(40 0) )! !

!PayrollSystemTestCase methodsFor: 'Testing' stamp: ''!
testAddEmployee
	myPayroll addEmployee: susan.
	self
		assert: (myPayroll employeeNames includes: 'Susan').
	myPayroll addEmployee: george.
	self assert: (myPayroll employeeNamed: 'Susan')
			= susan! !

!PayrollSystemTestCase methodsFor: 'Testing' stamp: 'Ajith Herga 1/30/2013 23:24'!
testEarnings
	myPayroll addEmployee: susan;
		 addEmployee: george;
		 addEmployee: ajith.
	self postTimeCards: normalMonth to: susan.
	self postTimeCards: aLittleOvertime  to: ajith.
	self postTimeCards: aLittleOvertime to: george.
	self assert: susan earned = 3200.
	self assert: george earned = 2100.
	self assert: ajith earned = 3675.0.! !

!PayrollSystemTestCase methodsFor: 'as yet unclassified' stamp: 'Ajith Herga 1/31/2013 11:02'!
testPayCheck
	"Check if the right paycheck is printing"

	john postTimeCardFor: day2 hoursWorked: 40 vacation: 0.
	john postTimeCardFor: (day2 addDays:  4) hoursWorked: 40 vacation: 0.1.
   (john makePaycheckAt: (day2 addDays: 14)) printOnCheckStream: (WriteStream on: (String new: 20)).! !

TestCase subclass: #SortedSequenceTestCase
	instanceVariableNames: 'sequence'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mayur-Sachin-Ajith'!

!SortedSequenceTestCase methodsFor: 'Running' stamp: ''!
setUp
	sequence := SortedSequence new.
	sequence add: 1;
		 add: 80;
		 add: 500;
		 add: 501! !

!SortedSequenceTestCase methodsFor: 'Testing' stamp: ''!
testIndex
	self assert: (sequence indexOfStartOfIntervalContaining: 1)
			= 1.
	self assert: (sequence indexOfStartOfIntervalContaining: 79)
			= 1.
	self assert: (sequence indexOfStartOfIntervalContaining: 80)
			= 2.
	self assert: (sequence indexOfStartOfIntervalContaining: 501)
			= 4! !

TestCase subclass: #UnitsTestCase
	instanceVariableNames: 'meters seconds testUnit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mayur-Sachin-Ajith'!

!UnitsTestCase methodsFor: 'running' stamp: 'Ajith Herga 1/31/2013 15:02'!
setUp 
	"comment stating purpose of message"
   testUnit := Units addUnits: 1 at: #seconds.! !

!UnitsTestCase methodsFor: 'as yet unclassified' stamp: 'Ajith Herga 1/31/2013 15:04'!
testInsert
	"test insert a value and check"
   testUnit at: #seconds ifPresent: [:x | Transcript show: x].! !
Employee initialize!
Paycheck initialize!
